#!/usr/bin/perl --
#
#----------------------------------------------------------------------------\
# NetReg 1.3rc2
# Copyright (C) 1999-2002 Peter Valian, Southwestern University
#
# register.cgi -- Student Registration Frontend
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Peter Valian <valianp@southwestern.edu>
# http://www.netreg.org/
#
# Code contributions by:
#
#   Bobby Clark <bobby.clark@eku.edu>, Eastern Kentucky University 
#     IMAP Authentication
#
#----------------------------------------------------------------------------/

require "variables.pl";
require "subnet.pl";

#----------------------------------------------------------------------------\
# Main Function
#----------------------------------------------------------------------------\
parse_subnets();
get_input();
check_input();
authenticate();
get_host_info();
append_host_entry();
print_success();
#----------------------------------------------------------------------------/


#############################################################################\
## SUBROUTINES BEGIN
#############################################################################/


#----------------------------------------------------------------------------\
# Get the input from the CGI FORM
#----------------------------------------------------------------------------\
sub get_input {
    my $buffer;
    read(STDIN, $buffer, $ENV{'CONTENT_LENGTH'});
    @pairs = split(/&/, $buffer);
    foreach $pair (@pairs) {
	($name, $value) = split(/=/, $pair);
	$value =~ tr/+/ /;
	$value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
	$value =~ s/\s+//g;
	$FORM{$name} = $value;
    }
}
#----------------------------------------------------------------------------/

#----------------------------------------------------------------------------\
# Make sure the user is allowed to register and
# we have all the input we need from the user
#----------------------------------------------------------------------------\
sub check_input {
    my $subnet;
    my $found = 0;
    foreach $subnet (values %URSUBS) {
	if (ip_in_subnet($IP,$subnet)) {
	    $found = 1;
	    last;
	}
    }

    if (!$found) {
	error("Permission Denied<P>You are not in my allowable IP range<br>$IP<br>$subnet");
    }

    if ((!$FORM{'user'}) || (!$FORM{'pass'})) {
      error("Invalid Login");
    }
}
#----------------------------------------------------------------------------/

#----------------------------------------------------------------------------\
# Use the POP or FTP server for user authentication
# POPStat will return the number of messages in user's INBOX or 
# will return -1 if the connection failed, $ftp->pwd is return null if the
# connection failed.
#----------------------------------------------------------------------------\
sub authenticate {
    if ($AUTH_METHOD eq "POP") {
	$pop=new Mail::POP3Client("$FORM{'user'}","$FORM{'pass'}","$POPSERVER");
	$alive=$pop->POPStat;
	$popmsg=$pop->Message;
	$pop->Close;
	unless ($alive ne "-1") {
	    error("Authentication Failed<P>POP replied:<br>$popmsg");
	}
    } elsif ($AUTH_METHOD eq "FTP") {
	$ftp = Net::FTP->new($FTPSERVER);
	$ftp->login($FORM{'user'}, $FORM{'pass'});
	$ftp->pwd()||error("Authentication Failed<P>Bad Username or Password.");
    } elsif ($AUTH_METHOD eq "IMAP") {
	$imap = new Net::IMAP::Simple( "$IMAPSERVER" );
	$alive = $imap->login( $FORM{'user'}, $FORM{'pass'} );
	$imap->quit();
	unless ($alive) {
	    error("Authentication Failed");
	}
    } else {
	error("No Authentication Method");
    }
}
#----------------------------------------------------------------------------/

#----------------------------------------------------------------------------\
# The User has authenticated, get their network & hardware info
#----------------------------------------------------------------------------\
sub get_host_info {
    # Parse through the leases file to get user MAC address based on IP 
    $DONE = 1;
    open(LEASES, "$LEASESPATH/$LEASESFILE") or 
	error( "Could not open lease file: $LEASESPATH/$LEASESFILE" );
    while (<LEASES>) {
	if (!$DONE) {
	    if ($_ =~ /ethernet/) {
		($TRASH, $TRASH, $TRASH, $MAC) = split / +|\;/;
		$RECORDS{$LEASEIP} = uc($MAC);
	    }
	    if ($_ =~ /\}/) { $DONE = 1; }
	} elsif (($_ =~ /\{/) && ($_ !~ /\#/)) {
	    ($TRASH, $LEASEIP) = split /\s/;
	    $DONE = 0;
	}
    }
    close (LEASES);
    if (!exists ($RECORDS{$IP})) {
	error("No Lease Found");
    }
}
#----------------------------------------------------------------------------/

#----------------------------------------------------------------------------\
# Put the date in a usable/sortable format (Y2K Compliant)
# Open dhcpd.conf.new in append mode and add the user as a known host
#----------------------------------------------------------------------------\
  sub append_host_entry {
    # Get the time and put it in the format we want
    ($SEC,$MIN,$HOUR,$DAY,$MO,$YR) = localtime;
    $YR+=1900; $MO++;
    if ($DAY<10) { $DAY="0$DAY"; }
    if ($MO<10) { $MO="0$MO"; }
    if ($HOUR<10) { $HOUR="0$HOUR"; }
    if ($MIN<10) { $MIN="0$MIN"; }
    if ($SEC<10) { $SEC="0$SEC"; }
    $TIME="$HOUR:$MIN:$SEC";

    @RECORDS = (keys %RECORDS);
    # Check if the MAC is already registered
    open (DHCPDCONFNEW, "$DHCPDCONFPATH/$DHCPDCONFFILE.new") or
	error( "Could not open new dhcp configuration file: $DHCPDCONFPATH/$DHCPDCONFFILE.new" );
    while (<DHCPDCONFNEW>) {
      if ($_ =~ /$RECORDS{$IP}/) {
        if ($RECORDS{$IP} eq "") { error("No Registration Required"); }
        else {
          error("Hardware Already Registered<br>Please Reboot Your Computer");
        }
      }
      if (/host $FORM{'user'}-/) {
        ($TRASH, $HOST) = split /\s/;
        push (@HOSTS, $HOST);
      }
    }
    close(DHCPDCONFNEW);

    my @SHOSTS = (sort { $a <=> $b } @HOSTS);
    ($TRASH, $SERIAL) = split(/-/,$SHOSTS[$#SHOSTS]);
    $SERIAL++;

    open (DHCPDCONFNEW, ">>$DHCPDCONFPATH/$DHCPDCONFFILE.new") or
	error( "Could not write new dhcp configuration file: $DHCPDCONFPATH/$DHCPDCONFFILE.new" );

    print DHCPDCONFNEW "host $FORM{'user'}-$SERIAL { hardware ethernet ";
    print DHCPDCONFNEW "$RECORDS{$IP} \; ";

    # Add DHCP option to use derived name in DDNS updates, if desired
    if ($DDNS_USE_HOST_DECL) {

	print DHCPDCONFNEW "ddns-hostname $FORM{'user'}-$SERIAL;";
    }

    print DHCPDCONFNEW "}\#$ENV{'HTTP_USER_AGENT'}";
    print DHCPDCONFNEW "\#$YR$MO$DAY $TIME\#$IP\n";
    close DHCPDCONFNEW;
  }
#----------------------------------------------------------------------------/


#----------------------------------------------------------------------------\
# The user has been added, print out success message
#----------------------------------------------------------------------------\
  sub print_success {
    print_custom_header();
    print "Registration Complete.<p>Please reboot your machine.<P>";
    print_custom_footer();
    exit;
  }
#----------------------------------------------------------------------------/

#----------------------------------------------------------------------------\
# Print out the error message and write it to the log file
#----------------------------------------------------------------------------\
  sub error {
    $ERROR_MSG = shift(@_);
    ($SEC,$MIN,$HOUR,$DAY,$MO,$YR) = localtime;
    $YR+=1900; $MO++;
    if ($HOUR<10) { $HOUR="0$HOUR"; }
    if ($MIN<10) { $MIN="0$MIN"; }
    if ($SEC<10) { $SEC="0$SEC"; }
    $TIME="$HOUR:$MIN:$SEC";
    if ($DAY<10) { $DAY="0$DAY"; }
    if ($MO<10) { $MO="0$MO"; }
    open (ERROR_LOG, ">>$ERROR_LOG");
    print ERROR_LOG "$FORM{'user'} [$YR$MO$DAY $TIME]: $ERROR_MSG\n";
    print ERROR_LOG "\$IP = $IP\n";
    print ERROR_LOG "\$RECORDS{\$IP} = $RECORDS{$IP}\n\n";

    print_custom_header();
    print "<TABLE WIDTH=350 border=0 cellspacing=0 cellpadding=1><TR>";
    print "<TH BGCOLOR=black><table width=348 cellpadding=5";
    print "cellspacing=0 align=center><TR><TH bgcolor=black>";
    print "<font color=white size=5><b>Registration Error</b></FONT></th>";
    print "</tr><tr><th bgcolor=\#DDD7FF>$ERROR_MSG</center><P>";
    print "Please try to register again.  If the problem persists, call the ";
    print "HelpDesk at x6570.<P>";
    print "</td></tr></table></th></tr></table>";
    print_custom_footer();
    exit;
  }
#----------------------------------------------------------------------------/

