# subnet.pl
# Functions and variables specific to the handling of subnets in CIDR notation
# for NetReg
#

# Build hash of all possible subnet masks
$max = 2**32;
for ($i=0; $i<32; $i++) {
    $masks{32-$i} = $max - 2**$i;
}

# Parse the subnet.dat file and load data into hashes or arrays

sub parse_subnets {

    open(SUBNETDAT, "$SUBNETFILE") or 
      error( "Cannot open subnet.dat file: $SUBNETFILE"),
      print_footer(),
      exit 0;

    while (<SUBNETDAT>) {
	# If the line doesn't begin with a digit, ignore it
	if (/^\d/) {
	    chomp;
	    $_ .= ':';
	    ($SUBNET, $LOC, $SUBIPS, $URSUB) = split /:/;
	    my $test = bad_subnet( $SUBNET );
	    if ($test) {
		error( $test, "Config" );
	    } else {
		$LOCS{$SUBNET} .= $LOC;
		$SUBIPS{$SUBNET} = $SUBIPS - 0;
		$MASKS{$SUBNET} = $MASK;
		$SUBIPS{$SUBNET} = $SUBIPS;
		if ($URSUB eq '') {
		    $URSUB = $SUBNET;
		} else {
		    $test = bad_subnet( $URSUB );
		    if ($test) {
			error( $test, "Config" );
		    }
		}
		$URSUBS{$SUBNET} = $URSUB;
	    }
	}
    }
    close (SUBNETDAT);
}

# Test if subnet/mask pair is valid
#   Return null if fine, other text of error with corrected subnet
#   address for the given mask.

sub bad_subnet {
    my ($subnet,$mask) = split('/', $_[0]);
    my $psn = ip2dec($subnet);
    my $retval;
    my $real_sn;
    my @rsn;
    my $i;

    if (($psn & $masks{$mask}) != $psn) {
	$retval = "Bad subnet $subnet for provided mask /$mask: should be ";
	$real_sn = ($psn & $masks{$mask});
	for ($i=3; $i>=0; $i--) {
	    $rsn[$i] = $real_sn & 255;
	    $real_sn = $real_sn >> 8;
	}
	$retval .= (join('.', @rsn) . "/$mask");
    }
    return $retval;
}

# Return true or false depending on whether an IP address is in the subnet provided

sub ip_in_subnet {
    my $ip = $_[0];
    my ($subnet,$mask) = split('/', $_[1]);
    my $pip = ip2dec($ip);
    my $psn = ip2dec($subnet);

    if ( ($pip & $masks{$mask}) == ($psn & $masks{$mask}) ) {
	return 1;    # IP address is in that subnet!
    } else {
	return 0;
    }
}


# Convert dotted-quad IP address to 32-bit int
sub ip2dec {
    ($o1,$o2,$o3,$o4) = split(/\./, $_[0]);
    ($o1 << 24) + ($o2 << 16) + ($o3 << 8) + $o4;
}

# For debugging only -- convert decimal to binary
sub dec2bin {
    return unpack("B32", pack("N", shift));
}

# For debugging only -- convert binary to decimal
sub bin2dec {
    return unpack("N", pack("B32", substr("0" x 32 . shift, -32)));
}

1;
